#define input1 	2
#define input2 	3
#define picture1 	4
#define NL1 	5
#define NL2 	6
#define start1 	7
#define start2 	8
#define start3 	9
#define elementlist1 	10
#define elementlist2 	11
#define elementlist3 	12
#define term1 	13
#define term2 	14
#define term3 	15
#define term4 	16
#define element1 	17
#define element2 	18
#define element3 	19
#define element4 	20
#define element5 	21
#define element6 	22
#define element7 	23
#define element8 	24
#define element9 	25
#define element10 	26
#define element11 	27
#define element12 	28
#define lbrace1 	29
#define namedobj1 	30
#define namedobj2 	31
#define suffix1 	32
#define suffix2 	33
#define suffix3 	34
#define position1 	35
#define position2 	36
#define position3 	37
#define position4 	38
#define assignlist1 	39
#define assignlist2 	40
#define command1 	41
#define command2 	42
#define command3 	43
#define command4 	44
#define command5 	45
#define command6 	46
#define command7 	47
#define command8 	48
#define command9 	49
#define command10 	50
#define command11 	51
#define optnl1 	52
#define optnl2 	53
#define ifpart1 	54
#define elsehead1 	55
#define for1 	56
#define for2 	57
#define stringexpr1 	58
#define stringexpr2 	59
#define string1 	60
#define string2 	61
#define assignment1 	62
#define assignment2 	63
#define assignrhs1 	64
#define assignrhs2 	65
#define expression1 	66
#define expression2 	67
#define expression3 	68
#define expression4 	69
#define expression5 	70
#define ifhead1 	71
#define setlogic1 	72
#define logexpr1 	73
#define logexpr2 	74
#define forhead1 	75
#define forincr1 	76
#define do1 	77
#define do2 	78
#define by1 	79
#define by2 	80
#define redirect1 	81
#define redirect2 	82
#define redirect3 	83
#define resetlist1 	84
#define resetlist2 	85
#define systemcmd1 	86
#define sprintf1 	87
#define sprintf2 	88
#define exprlist1 	89
#define exprlist2 	90
#define object1 	91
#define object2 	92
#define object3 	93
#define object4 	94
#define object5 	95
#define object6 	96
#define object7 	97
#define object8 	98
#define object9 	99
#define object10 	100
#define object11 	101
#define object12 	102
#define object13 	103
#define object14 	104
#define object15 	105
#define object16 	106
#define object17 	107
#define object18 	108
#define object19 	109
#define object20 	110
#define object21 	111
#define object22 	112
#define object23 	113
#define object24 	114
#define object25 	115
#define object26 	116
#define object27 	117
#define openblock1 	118
#define block1 	119
#define block2 	120
#define block3 	121
#define block4 	122
#define optexp1 	123
#define optexp2 	124
#define closeblock1 	125
#define objectwith1 	126
#define objectwith2 	127
#define objectwith3 	128
#define pair1 	129
#define pair2 	130
#define nth1 	131
#define nth2 	132
#define nth3 	133
#define primobj1 	134
#define primobj2 	135
#define primobj3 	136
#define primobj4 	137
#define shift1 	138
#define shift2 	139
#define shift3 	140
#define location1 	141
#define location2 	142
#define location3 	143
#define location4 	144
#define location5 	145
#define place1 	146
#define place2 	147
#define place3 	148
#define place4 	149
#define place5 	150
#define factor1 	151
#define factor2 	152
#define factor3 	153
#define placename1 	154
#define placename2 	155
#define placename3 	156
#define placename4 	157
#define ncount1 	158
#define ncount2 	159
#define ncount3 	160
#define logprod1 	161
#define logprod2 	162
#define logval1 	163
#define logval2 	164
#define logval3 	165
#define lcompare1 	166
#define lcompare2 	167
#define lcompare3 	168
#define lcompare4 	169
#define primary1 	170
#define primary2 	171
#define primary3 	172
#define primary4 	173
#define primary5 	174
#define primary6 	175
#define primary7 	176
#define primary8 	177
#define primary9 	178
#define primary10 	179
#define primary11 	180
#define primary12 	181
#define primary13 	182
