
%token	XEMPTY 1 "blank"
%token	Xlt 2 "<"
%token	Xcw 3 "cw"
%token	Xccw 4 "ccw"
%token	Xlparen 5 "("
%token	Xrparen 6 ")"
%token	Xmult 7 "*"
%token	Xplus 8 "+"
%token	Xminus 9 "-"
%token	Xdiv 10 "/"
%token	Xpercent 11 "%"
%token	Xnewline 12 "end of line or ;"
%token	Xcaret 13 "^"
%token	Xnot 14 "!"
%token	Xandand 15 "&&"
%token	Xoror 16 "||"
%token	Xcomma 17 ","
%token	Xcolon 18 ":"
%token	Xleftbrack 19 "["
%token	Xrightbrack 20 "]"
%token	Xleftbrace 21 "{"
%token	Xrightbrace 22 "}"
%token	Xdot 23 "."
%token	Xblock 24 "[]"
%token	Xleftquote 25 "`"
%token	Xrightquote 26 "'"
%token	Xeq 27 "="
%token	Xcoloneq 28 ":="
%token	Xpluseq 29 "+="
%token	Xminuseq 30 "-="
%token	Xmulteq 31 "*="
%token	Xdiveq 32 "/="
%token	Xremeq 33 "%="
%token	Xampersand 34 "&"
%token	Xfloat 35 "number"
%token	Xname 36 "identifier"
%token	Xlabel 37 "label"
%token	XLaTeX 38 "LaTeX"
%token	Xstring 39 "string"
%token	Xcomment 40 "#"
%token	Xarg 41 "$ argument"
%token	DotPS 42 ".PS"
%token	DotPE 43 "end of file or .PE"
%token	Xht 44 "ht"
%token	Xwid 45 "wid"
%token	Xrad 46 "rad"
%token	Xdiam 47 "diam"
%token	Xthick 48 "thick"
%token	Xscaled 49 "scaled"
%token	Xfrom 50 "from"
%token	Xto 51 "to"
%token	Xat 52 "at"
%token	Xwith 53 "with"
%token	Xby 54 "by"
%token	Xthen 55 "then"
%token	Xcontinue 56 "continue"
%token	Xchop 57 "chop"
%token	Xsame 58 "same"
%token	Xof 59 "of"
%token	Xthe 60 "the"
%token	Xway 61 "way"
%token	Xbetween 62 "between"
%token	Xand 63 "and"
%token	XHere 64 "Here"
%token	Xnth 65 "ordinal: st or rd or nd or th"
%token	Xlast 66 "last"
%token	Xfill 67 "fill"
%token	Dotx 68 ".x"
%token	Doty 69 ".y"
%token	Xprint 70 "print"
%token	Xcopy 71 "copy"
%token	Xreset 72 "reset"
%token	Xexec 73 "exec"
%token	Xsh 74 "sh"
%token	Xcommand 75 "command"
%token	Xdefine 76 "define"
%token	Xundefine 77 "undef"
%token	Xrand 78 "rand"
%token	Xif 79 "if"
%token	Xelse 80 "else"
%token	Xfor 81 "for"
%token	Xdo 82 "do"
%token	Xendfor 83 "end of for {...} contents"
%token	Xsprintf 84 "sprintf"
%token	Xcorner 85 "compass corner: .n .ne .center .end etc"
%token	Dotne 86 ".ne"
%token	Dotse 87 ".se"
%token	Dotnw 88 ".nw"
%token	Dotsw 89 ".sw"
%token	Dotn 90 ".top or .n"
%token	Dots 91 ".bot or .s"
%token	Dote 92 ".right or .e"
%token	Dotw 93 ".left or .w"
%token	Dotstart 94 "start"
%token	Dotend 95 "end"
%token	Dotc 96 ".c"
%token	Xcompare 97 "logical operator"
%token	Xeqeq 98 "=="
%token	Xneq 99 "!="
%token	Xge 100 ">="
%token	Xle 101 "<="
%token	Xgt 102 ">"
%token	Xparam 103 "attribute .ht .wid etc"
%token	Xheight 104 ".ht"
%token	Xwidth 105 ".wid"
%token	Xradius 106 ".rad"
%token	Xdiameter 107 ".diam"
%token	Xthickness 108 ".thick"
%token	Xlength 109 ".len"
%token	Xfunc1 110 "function (1 arg)"
%token	Xabs 111 "abs"
%token	Xacos 112 "acos"
%token	Xasin 113 "asin"
%token	Xcos 114 "cos"
%token	Xexp 115 "exp"
%token	Xexpe 116 "expe"
%token	Xint 117 "int"
%token	Xlog 118 "log"
%token	Xloge 119 "loge"
%token	Xsign 120 "sign"
%token	Xsin 121 "sin"
%token	Xsqrt 122 "sqrt"
%token	Xtan 123 "tan"
%token	Xfloor 124 "floor"
%token	Xfunc2 125 "function (2 args)"
%token	Xatan2 126 "atan2"
%token	Xmax 127 "max"
%token	Xmin 128 "min"
%token	Xpmod 129 "pmod"
%token	Xlinetype 130 "line type: dotted etc"
%token	Xsolid 131 "solid"
%token	Xdotted 132 "dotted"
%token	Xdashed 133 "dashed"
%token	Xinvis 134 "invisible"
%token	Xcolrspec 135 "color spec: colored outlined shaded"
%token	Xcolour 136 "coloured"
%token	Xoutlined 137 "outlined"
%token	Xshaded 138 "shaded"
%token	Xtextpos 139 "text position: ljust rjust above below center"
%token	Xcenter 140 "center"
%token	Xljust 141 "ljust"
%token	Xrjust 142 "rjust"
%token	Xabove 143 "above"
%token	Xbelow 144 "below"
%token	Xarrowhd 145 "arrowhead parameter: <- -> <->"
%token	Xlefthead 146 "<-"
%token	Xrighthead 147 "->"
%token	Xdoublehead 148 "<->"
%token	Xdirecton 149 "direction: up down right left"
%token	Xup 150 "up"
%token	Xdown 151 "down"
%token	Xright 152 "right"
%token	Xleft 153 "left"
%token	Xprimitiv 154 "drawn object: box circle line etc"
%token	Xbox 155 "box"
%token	Xcircle 156 "circle"
%token	Xellipse 157 "ellipse"
%token	Xarc 158 "arc"
%token	Xline 159 "line"
%token	Xarrow 160 "arrow"
%token	Xmove 161 "move"
%token	Xspline 162 "spline"
%token	Xenvvar 163 "environment variable"
%token	Xarcrad 164 "arcrad"
%token	Xarrowht 165 "arrowht"
%token	Xarrowwid 166 "arrowwid"
%token	Xboxht 167 "boxht"
%token	Xboxrad 168 "boxrad"
%token	Xboxwid 169 "boxwid"
%token	Xcirclerad 170 "circlerad"
%token	Xdashwid 171 "dashwid"
%token	Xellipseht 172 "ellipseht"
%token	Xellipsewid 173 "ellipsewid"
%token	Xlineht 174 "lineht"
%token	Xlinewid 175 "linewid"
%token	Xmoveht 176 "moveht"
%token	Xmovewid 177 "movewid"
%token	Xtextht 178 "textht"
%token	Xtextoffset 179 "textoffset"
%token	Xtextwid 180 "textwid"
%token	Xarrowhead 181 "arrowhead"
%token	Xfillval 182 "fillval"
%token	Xlinethick 183 "linethick"
%token	Xmaxpsht 184 "maxpsht"
%token	Xmaxpswid 185 "maxpswid"
%token	Xscale 186 "scale"
