
#define	XEMPTY	1
#define	Xlt	2
#define	Xcw	3
#define	Xccw	4
#define	Xlparen	5
#define	Xrparen	6
#define	Xmult	7
#define	Xplus	8
#define	Xminus	9
#define	Xdiv	10
#define	Xpercent	11
#define	Xnewline	12
#define	Xcaret	13
#define	Xnot	14
#define	Xandand	15
#define	Xoror	16
#define	Xcomma	17
#define	Xcolon	18
#define	Xleftbrack	19
#define	Xrightbrack	20
#define	Xleftbrace	21
#define	Xrightbrace	22
#define	Xdot	23
#define	Xblock	24
#define	Xleftquote	25
#define	Xrightquote	26
#define	Xeq	27
#define	Xcoloneq	28
#define	Xpluseq	29
#define	Xminuseq	30
#define	Xmulteq	31
#define	Xdiveq	32
#define	Xremeq	33
#define	Xampersand	34
#define	Xfloat	35
#define	Xname	36
#define	Xlabel	37
#define	XLaTeX	38
#define	Xstring	39
#define	Xcomment	40
#define	Xarg	41
#define	DotPS	42
#define	DotPE	43
#define	Xht	44
#define	Xwid	45
#define	Xrad	46
#define	Xdiam	47
#define	Xthick	48
#define	Xscaled	49
#define	Xfrom	50
#define	Xto	51
#define	Xat	52
#define	Xwith	53
#define	Xby	54
#define	Xthen	55
#define	Xcontinue	56
#define	Xchop	57
#define	Xsame	58
#define	Xof	59
#define	Xthe	60
#define	Xway	61
#define	Xbetween	62
#define	Xand	63
#define	XHere	64
#define	Xnth	65
#define	Xlast	66
#define	Xfill	67
#define	Dotx	68
#define	Doty	69
#define	Xprint	70
#define	Xcopy	71
#define	Xreset	72
#define	Xexec	73
#define	Xsh	74
#define	Xcommand	75
#define	Xdefine	76
#define	Xundefine	77
#define	Xrand	78
#define	Xif	79
#define	Xelse	80
#define	Xfor	81
#define	Xdo	82
#define	Xendfor	83
#define	Xsprintf	84
#define	Xcorner	85
#define	Dotne	86
#define	Dotse	87
#define	Dotnw	88
#define	Dotsw	89
#define	Dotn	90
#define	Dots	91
#define	Dote	92
#define	Dotw	93
#define	Dotstart	94
#define	Dotend	95
#define	Dotc	96
#define	Xcompare	97
#define	Xeqeq	98
#define	Xneq	99
#define	Xge	100
#define	Xle	101
#define	Xgt	102
#define	Xparam	103
#define	Xheight	104
#define	Xwidth	105
#define	Xradius	106
#define	Xdiameter	107
#define	Xthickness	108
#define	Xlength	109
#define	Xfunc1	110
#define	Xabs	111
#define	Xacos	112
#define	Xasin	113
#define	Xcos	114
#define	Xexp	115
#define	Xexpe	116
#define	Xint	117
#define	Xlog	118
#define	Xloge	119
#define	Xsign	120
#define	Xsin	121
#define	Xsqrt	122
#define	Xtan	123
#define	Xfloor	124
#define	Xfunc2	125
#define	Xatan2	126
#define	Xmax	127
#define	Xmin	128
#define	Xpmod	129
#define	Xlinetype	130
#define	Xsolid	131
#define	Xdotted	132
#define	Xdashed	133
#define	Xinvis	134
#define	Xcolrspec	135
#define	Xcolour	136
#define	Xoutlined	137
#define	Xshaded	138
#define	Xtextpos	139
#define	Xcenter	140
#define	Xljust	141
#define	Xrjust	142
#define	Xabove	143
#define	Xbelow	144
#define	Xarrowhd	145
#define	Xlefthead	146
#define	Xrighthead	147
#define	Xdoublehead	148
#define	Xdirecton	149
#define	Xup	150
#define	Xdown	151
#define	Xright	152
#define	Xleft	153
#define	Xprimitiv	154
#define	Xbox	155
#define	Xcircle	156
#define	Xellipse	157
#define	Xarc	158
#define	Xline	159
#define	Xarrow	160
#define	Xmove	161
#define	Xspline	162
#define	Xenvvar	163
#define	Xarcrad	164
#define	Xarrowht	165
#define	Xarrowwid	166
#define	Xboxht	167
#define	Xboxrad	168
#define	Xboxwid	169
#define	Xcirclerad	170
#define	Xdashwid	171
#define	Xellipseht	172
#define	Xellipsewid	173
#define	Xlineht	174
#define	Xlinewid	175
#define	Xmoveht	176
#define	Xmovewid	177
#define	Xtextht	178
#define	Xtextoffset	179
#define	Xtextwid	180
#define	Xarrowhead	181
#define	Xfillval	182
#define	Xlinethick	183
#define	Xmaxpsht	184
#define	Xmaxpswid	185
#define	Xscale	186
